"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var errors_1 = require("./errors");
var spying_1 = require("./spying");
var matchers_1 = require("./matchers");
/**
 * Allows checking of test outcomes
 * @param actualValue - the value or function under test
 */
function Expect(actualValue) {
    return new Matcher(actualValue);
}
exports.Expect = Expect;
/**
 * Gives functionality to ensure the outcome of a test is as expected
 */
var Matcher = (function () {
    function Matcher(actualValue) {
        this._shouldMatch = true;
        this._actualValue = actualValue;
    }
    Object.defineProperty(Matcher.prototype, "actualValue", {
        get: function () {
            return this._actualValue;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Matcher.prototype, "shouldMatch", {
        get: function () {
            return this._shouldMatch;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Matcher.prototype, "not", {
        /**
         * Any subsequent matcher function will be looking for the opposite criteria
         */
        get: function () {
            this._shouldMatch = !this.shouldMatch;
            return this;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Checks that a value is identical to another
     * @param expectedValue - the value that will be used to match
     */
    Matcher.prototype.toBe = function (expectedValue) {
        if (expectedValue !== this._actualValue === this.shouldMatch) {
            throw new errors_1.ExactMatchError(this._actualValue, expectedValue, this.shouldMatch);
        }
    };
    /**
     * Checks that a value is equal to another (for objects the function will check for deep equality)
     * @param expectedValue - the value that will be used to match
     */
    Matcher.prototype.toEqual = function (expectedValue) {
        // exclude the double equals in this case from review as this is what we want to do
        if (expectedValue != this._actualValue === this.shouldMatch) {
            if (typeof expectedValue !== "object" ||
                this._checkObjectsAreDeepEqual(expectedValue, this._actualValue) !== this.shouldMatch) {
                throw new errors_1.EqualMatchError(this._actualValue, expectedValue, this.shouldMatch);
            }
        }
    };
    /**
     * Checks that a value conforms to a regular expression
     * @param regex - the regular expression that the actual value should match
     */
    Matcher.prototype.toMatch = function (regex) {
        if (regex === null || regex === undefined) {
            throw new TypeError("toMatch regular expression must not be null or undefined.");
        }
        if (typeof this._actualValue !== "string") {
            throw new TypeError("toMatch must only be used to match on strings.");
        }
        if (!regex.test(this._actualValue) === this.shouldMatch) {
            throw new errors_1.RegexMatchError(this._actualValue, regex, this.shouldMatch);
        }
    };
    /**
     * Checks that a value is not undefined
     */
    Matcher.prototype.toBeDefined = function () {
        if (this._actualValue === undefined === this.shouldMatch) {
            throw new errors_1.ExactMatchError(this._actualValue, undefined, !this.shouldMatch);
        }
    };
    /**
     * Checks that a value is null
     */
    Matcher.prototype.toBeNull = function () {
        if (this._actualValue !== null === this.shouldMatch) {
            throw new errors_1.ExactMatchError(this._actualValue, null, this.shouldMatch);
        }
    };
    /**
     * Checks that a value is equivalent to boolean true
     */
    Matcher.prototype.toBeTruthy = function () {
        if ((this._actualValue && !this.shouldMatch) || (!this._actualValue && this.shouldMatch)) {
            throw new errors_1.TruthyMatchError(this._actualValue, this.shouldMatch);
        }
    };
    /**
     * Checks that a string contains another string or an array contains a specific item
     * @param expectedContent - the string or array item that the value should contain
     */
    Matcher.prototype.toContain = function (expectedContent) {
        if (this._actualValue instanceof Array === false && typeof this._actualValue !== "string") {
            throw new TypeError("toContain must only be used to check whether strings or arrays contain given contents.");
        }
        if (typeof this._actualValue === "string" && typeof expectedContent !== "string") {
            throw new TypeError("toContain cannot check whether a string contains a non string value.");
        }
        if (this._actualValue.indexOf(expectedContent) === -1 === this.shouldMatch) {
            throw new errors_1.ContentsMatchError(this._actualValue, expectedContent, this.shouldMatch);
        }
    };
    /**
     * Checks that a number is less than a given limit
     * @param upperLimit - the number that the number under test should be less than
     */
    Matcher.prototype.toBeLessThan = function (upperLimit) {
        if (upperLimit === null || upperLimit === undefined) {
            throw new TypeError("toBeLessThan upper limit must not be null or undefined.");
        }
        if (typeof this._actualValue !== "number") {
            throw new TypeError("toBeLessThan can only check numbers.");
        }
        if (this._actualValue < upperLimit !== this.shouldMatch) {
            throw new errors_1.LessThanMatchError(this._actualValue, upperLimit, this.shouldMatch);
        }
    };
    /**
     * Checks that a number is greater than a given limit
     * @param lowerLimit - the number that the number under test should be greater than
     */
    Matcher.prototype.toBeGreaterThan = function (lowerLimit) {
        if (lowerLimit === null || lowerLimit === undefined) {
            throw new TypeError("toBeGreaterThan lower limit must not be null or undefined.");
        }
        if (typeof this._actualValue !== "number") {
            throw new TypeError("toBeGreaterThan can only check numbers.");
        }
        if (this._actualValue > lowerLimit !== this.shouldMatch) {
            throw new errors_1.GreaterThanMatchError(this._actualValue, lowerLimit, this.shouldMatch);
        }
    };
    /**
     * Checks that an array is empty, a string is empty, or an object literal has no properties
     */
    Matcher.prototype.toBeEmpty = function () {
        if (null === this.actualValue || undefined === this.actualValue) {
            throw new TypeError("toBeEmpty requires value passed in to Expect not to be null or undefined");
        }
        if (typeof this.actualValue === "string" || Array.isArray(this.actualValue)) {
            if ((this.actualValue.length === 0) !== this.shouldMatch) {
                throw new errors_1.EmptyMatchError(this.actualValue, this.shouldMatch);
            }
        }
        else if (this.actualValue.constructor === Object) {
            if ((Object.keys(this.actualValue).length === 0) !== this.shouldMatch) {
                throw new errors_1.EmptyMatchError(this.actualValue, this.shouldMatch);
            }
        }
        else {
            throw new TypeError("toBeEmpty requires value passed in to Expect to be an array, string or object literal");
        }
    };
    /**
     * Checks that a function throws an error when executed
     */
    Matcher.prototype.toThrow = function () {
        if (this._actualValue instanceof Function === false) {
            throw new TypeError("toThrow requires value passed in to Expect to be a function.");
        }
        var errorThrown;
        try {
            this._actualValue();
        }
        catch (error) {
            errorThrown = error;
        }
        if (errorThrown === undefined === this.shouldMatch) {
            throw new errors_1.ErrorMatchError(errorThrown, this.shouldMatch);
        }
    };
    Matcher.prototype.toThrowAsync = function () {
        return __awaiter(this, void 0, void 0, function () {
            var errorThrown, error_1;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (this._actualValue instanceof Function === false) {
                            throw new TypeError("toThrowAsync requires value passed in to Expect to be a function.");
                        }
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 4]);
                        return [4 /*yield*/, this._actualValue()];
                    case 2:
                        _a.sent();
                        return [3 /*break*/, 4];
                    case 3:
                        error_1 = _a.sent();
                        errorThrown = error_1;
                        return [3 /*break*/, 4];
                    case 4:
                        if (errorThrown === undefined === this.shouldMatch) {
                            throw new errors_1.ErrorMatchError(errorThrown, this.shouldMatch);
                        }
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Checks that a function throws a specific error
     * @param errorType - the type of error that should be thrown
     * @param errorMessage - the message that the error should have
     */
    Matcher.prototype.toThrowError = function (errorType, errorMessage) {
        if (this._actualValue instanceof Function === false) {
            throw new TypeError("toThrowError requires value passed in to Expect to be a function.");
        }
        var threwRightError = false;
        var actualError;
        try {
            this._actualValue();
        }
        catch (error) {
            actualError = error;
            if (error instanceof errorType && error.message === errorMessage) {
                threwRightError = true;
            }
        }
        if (threwRightError !== this.shouldMatch) {
            throw new errors_1.ErrorMatchError(actualError, this.shouldMatch, errorType, errorMessage);
        }
    };
    /**
     * Checks that a function throws a specific error asynchronously
     * @param errorType - the type of error that should be thrown
     * @param errorMessage - the message that the error should have
     */
    Matcher.prototype.toThrowErrorAsync = function (errorType, errorMessage) {
        return __awaiter(this, void 0, void 0, function () {
            var threwRightError, actualError, error_2;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (this._actualValue instanceof Function === false) {
                            throw new TypeError("toThrowErrorAsync requires value passed to Expect to be a function.");
                        }
                        threwRightError = false;
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 4]);
                        return [4 /*yield*/, this._actualValue()];
                    case 2:
                        _a.sent();
                        return [3 /*break*/, 4];
                    case 3:
                        error_2 = _a.sent();
                        actualError = error_2;
                        if (error_2 instanceof errorType && error_2.message === errorMessage) {
                            threwRightError = true;
                        }
                        return [3 /*break*/, 4];
                    case 4:
                        if (threwRightError !== this.shouldMatch) {
                            throw new errors_1.ErrorMatchError(actualError, this.shouldMatch, errorType, errorMessage);
                        }
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Checks that a spy has been called
     */
    Matcher.prototype.toHaveBeenCalled = function () {
        if (this._isFunctionSpyOrSpiedOnFunction(this._actualValue) === false) {
            throw new TypeError("toHaveBeenCalled requires value passed in to Expect to be a FunctionSpy or a spied on function.");
        }
        if (this._actualValue.calls.length === 0 === this.shouldMatch) {
            throw new errors_1.FunctionCallMatchError(this._actualValue, this.shouldMatch);
        }
        return new matchers_1.FunctionSpyMatcher(this._actualValue);
    };
    /**
     * Checks that a spy has been called with the specified arguments
     * @param expectedArguments - a list of arguments that the spy should have been called with
     */
    Matcher.prototype.toHaveBeenCalledWith = function () {
        var expectedArguments = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            expectedArguments[_i] = arguments[_i];
        }
        if (this._isFunctionSpyOrSpiedOnFunction(this._actualValue) === false) {
            throw new TypeError("toHaveBeenCalledWith requires value passed in to Expect to be a FunctionSpy or a spied on function.");
        }
        if (this._actualValue.calls.filter(function (call) {
            return call.args.length === expectedArguments.length &&
                call.args.filter(function (arg, index) {
                    var expectedArgument = expectedArguments[index];
                    return arg === expectedArgument ||
                        expectedArgument === spying_1.Any ||
                        (expectedArgument instanceof spying_1.TypeMatcher && expectedArgument.test(arg));
                }).length === expectedArguments.length; // and all call arguments match expected arguments
        }).length === 0 === this.shouldMatch) {
            throw new errors_1.FunctionCallMatchError(this._actualValue, this.shouldMatch, expectedArguments);
        }
        return new matchers_1.FunctionSpyMatcher(this._actualValue, expectedArguments);
    };
    /**
     * Checks that a property spy has been set
     */
    Matcher.prototype.toHaveBeenSet = function () {
        if (this._actualValue instanceof spying_1.PropertySpy === false) {
            throw new TypeError("toHaveBeenSet requires value passed in to Expect to be a PropertySpy.");
        }
        if (this._actualValue.setCalls.length === 0 === this.shouldMatch) {
            throw new errors_1.PropertySetMatchError(this._actualValue, this.shouldMatch);
        }
    };
    /**
     * Checks that a property spy has been set to a specific value
     * @param value - a value to which the property should be set to
     */
    Matcher.prototype.toHaveBeenSetTo = function (value) {
        if (this._actualValue instanceof spying_1.PropertySpy === false) {
            throw new TypeError("toHaveBeenSetTo requires value passed in to Expect to be a PropertySpy.");
        }
        if (this._actualValue.setCalls.filter(function (call) { return call.args[0] === value; }).length === 0 === this.shouldMatch) {
            throw new errors_1.PropertySetMatchError(this._actualValue, this.shouldMatch, value);
        }
    };
    Matcher.prototype._isFunctionSpyOrSpiedOnFunction = function (value) {
        return value instanceof spying_1.FunctionSpy || (value instanceof Function && value.calls !== undefined);
    };
    Matcher.prototype._checkObjectsAreDeepEqual = function (objectA, objectB) {
        // if one object is an array and the other is not then they are not equal
        if (Array.isArray(objectA) !== Array.isArray(objectB)) {
            return false;
        }
        // get all the property keys for each object
        var objectAKeys = Object.keys(objectA);
        var objectBKeys = Object.keys(objectB);
        // if they don't have the same amount of properties then clearly not
        if (objectAKeys.length !== objectBKeys.length) {
            return false;
        }
        // check all the properties of each object
        for (var _i = 0, objectAKeys_1 = objectAKeys; _i < objectAKeys_1.length; _i++) {
            var objectAKey = objectAKeys_1[_i];
            // if the property values are not the same
            if (objectA[objectAKey] !== objectB[objectAKey]) {
                // and it's not an object or the objects are not equal
                if (typeof (objectA[objectAKey]) !== "object" ||
                    this._checkObjectsAreDeepEqual(objectA[objectAKey], objectB[objectAKey]) === false) {
                    // then not deep equal
                    return false;
                }
            }
        }
        // all properties match so all is good
        return true;
    };
    return Matcher;
}());
exports.Matcher = Matcher;
//# sourceMappingURL=expect.js.map