"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var errors_1 = require("../errors");
var _1 = require("./");
var FunctionSpyMatcher = (function () {
    function FunctionSpyMatcher(spy, expectedArguments) {
        this._expectedArguments = null;
        if (spy === null || spy === undefined) {
            throw new TypeError("spy must not be null or undefined.");
        }
        if (expectedArguments) {
            this._expectedArguments = expectedArguments;
        }
        this._spy = spy;
    }
    FunctionSpyMatcher.prototype.exactly = function (expectedCallCount) {
        if (expectedCallCount < 1) {
            throw new TypeError("expectedCallCount must be greater than 0.");
        }
        if (this._expectedArguments
            && this._getArgumentsMatching(this._expectedArguments).length !== expectedCallCount) {
            throw new errors_1.FunctionCallCountMatchError(this._spy, true, expectedCallCount, _1.SpyCallCountType.Exactly, this._expectedArguments);
        }
        else if (this._spy.calls.length !== expectedCallCount) {
            throw new errors_1.FunctionCallCountMatchError(this._spy, true, expectedCallCount, _1.SpyCallCountType.Exactly);
        }
        return new _1.FunctionSpyCallCountMatcher();
    };
    FunctionSpyMatcher.prototype.anythingBut = function (unexpectedCallCount) {
        if (unexpectedCallCount < 1) {
            throw new TypeError("unexpectedCallCount must be greater than 0.");
        }
        if (this._expectedArguments
            && this._getArgumentsMatching(this._expectedArguments).length === unexpectedCallCount) {
            throw new errors_1.FunctionCallCountMatchError(this._spy, false, unexpectedCallCount, _1.SpyCallCountType.Exactly, this._expectedArguments);
        }
        else if (this._spy.calls.length === unexpectedCallCount) {
            throw new errors_1.FunctionCallCountMatchError(this._spy, false, unexpectedCallCount, _1.SpyCallCountType.Exactly);
        }
        return new _1.FunctionSpyCallCountMatcher();
    };
    FunctionSpyMatcher.prototype.greaterThan = function (minimumCallCount) {
        if (minimumCallCount < 1) {
            throw new TypeError("minimumCallCount must be greater than 0.");
        }
        if (this._expectedArguments
            && this._getArgumentsMatching(this._expectedArguments).length <= minimumCallCount) {
            throw new errors_1.FunctionCallCountMatchError(this._spy, true, minimumCallCount, _1.SpyCallCountType.GreaterThan, this._expectedArguments);
        }
        else if (this._spy.calls.length <= minimumCallCount) {
            throw new errors_1.FunctionCallCountMatchError(this._spy, true, minimumCallCount, _1.SpyCallCountType.GreaterThan);
        }
        return new _1.FunctionSpyCallCountMatcher();
    };
    FunctionSpyMatcher.prototype.lessThan = function (maximumCallCount) {
        if (maximumCallCount < 1) {
            throw new TypeError("maximumCallCount must be greater than 0.");
        }
        if (this._expectedArguments
            && this._getArgumentsMatching(this._expectedArguments).length >= maximumCallCount) {
            throw new errors_1.FunctionCallCountMatchError(this._spy, true, maximumCallCount, _1.SpyCallCountType.LessThan, this._expectedArguments);
        }
        else if (this._spy.calls.length >= maximumCallCount) {
            throw new errors_1.FunctionCallCountMatchError(this._spy, true, maximumCallCount, _1.SpyCallCountType.LessThan);
        }
        return new _1.FunctionSpyCallCountMatcher();
    };
    FunctionSpyMatcher.prototype._getArgumentsMatching = function (expectedArguments) {
        return this._spy.callsWithArguments.apply(this._spy, expectedArguments);
    };
    return FunctionSpyMatcher;
}());
exports.FunctionSpyMatcher = FunctionSpyMatcher;
//# sourceMappingURL=function-spy-matcher.js.map